package fs.client;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ContentID {

        public static void main(String[] args) throws IOException, NoSuchAlgorithmException {
            // takes 2 arguments
            //1. Input file to convert into a merkle tree structure
            //2. Output directory to store the merkle tree directory structure (metadata to be stored on nodes)
            //3. The output directory of reconstructed file
            File file = new File(args[0]);
            double start = System.currentTimeMillis();
            byte[] byteArray;
            FileInputStream inputStream = new FileInputStream(file);
            byteArray = inputStream.readAllBytes();
            List<Chunk> chunks = getFileChunks(byteArray);
            Node root =MerkleTree.createMerkleTree(MerkleTree.createNodeList(chunks));
            double end = System.currentTimeMillis();
            System.out.println(root.getHash());
            double timeReq =  end-start/1000;
            System.out.println("Time required:" + timeReq);
            MerkleTree.printNAryTree(root,args[1]);
            reconstructFile(chunks, args[2]+"/reconstructed"+String.valueOf(System.currentTimeMillis()));
        }
    public static List<Chunk> getFileChunks(byte[] mainFile) throws NoSuchAlgorithmException {
        int kb = 4 * 1024;
        int endOfChunk=0;
        List<Chunk> chunks = new ArrayList<>();
        for (int i = 0; i < mainFile.length; ) {
            Chunk chunk = new Chunk();
            int min = Math.min(kb, mainFile.length - i);
            byte[] chunkContent = new byte[min];
            endOfChunk+= min;
            for (int j = 0; j < chunkContent.length; j++, i++) {
                chunkContent[j] = mainFile[i];
            }
            chunk.setContent(chunkContent);
            chunk.setHash(HashSHA256.createHashFromFileContent(chunkContent));
            chunk.setEndOfBlock(endOfChunk);
            chunks.add(chunk);
        }
        return chunks;
    }

    public static void reconstructFile(List<Chunk> chunks, String location) throws IOException {
        Path path = Paths.get(location);
        Files.createFile(path);
        Collections.sort(chunks, (c1,c2)-> (int) (c1.getEndOfBlock()-c2.getEndOfBlock()));
        for(Chunk chunk: chunks){
            Files.write(path, chunk.getContent(), StandardOpenOption.APPEND);
        }
    }
}
