package fs.client;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

public class MerkleTree {

    static List<Node> createNodeList(List<Chunk> chunksForTree){
        Collections.sort(chunksForTree, (c1,c2)-> (int) (c1.endOfBlock- c2.endOfBlock));
        List<Node> list = new LinkedList<>();
        for(Chunk chunk: chunksForTree){
            Node node = new Node(chunk.getHash(), chunk.getEndOfBlock(),true,chunk.getContent());
            list.add(node);
        }
        return list;
    }

    static Node createMerkleTree(List<Node> nodeList) throws NoSuchAlgorithmException {
        Node root;
        StringBuilder sb = new StringBuilder();
        List<Node> parentList = new LinkedList<>();
        int childCount=0;
        Node parent = null;
        if(nodeList.size()==1){
            nodeList.get(0).setRoot(true);
            return nodeList.get(0);
        }
        for(Node node: nodeList){
            if(childCount%128==0) {
                if(parent!=null) {
                    parent.setHash(HashSHA256.createHashFromFileContent(sb.toString().getBytes(StandardCharsets.UTF_8)));
                    parentList.add(parent);
                    sb = new StringBuilder();
                }
                parent = new Node();
            }
            parent.addChild(node);
            sb.append(node.getHash());
            childCount++;
            if(childCount==nodeList.size()) {
                parent.setHash(HashSHA256.createHashFromFileContent(sb.toString().getBytes(StandardCharsets.UTF_8)));
                parentList.add(parent);
            }
        }
        if(parentList.size()!=1){
            root = createMerkleTree(parentList);
        }else{
            parentList.get(0).setRoot(true);
            root = parentList.get(0);
        }
        return root;
    }

     static void printNAryTree(Node root, String outputDir) throws IOException {
        if(root == null) return;
        Queue<Node> queue = new LinkedList<>();
        queue.offer(root);
        while(!queue.isEmpty()) {
            int len = queue.size();
            for(int i=0;i<len;i++) { // so that we can reach each level
                Node node = queue.poll();
                System.out.print(node.getEndByte() + " ");
                if(node.getChildrenNodes()!=null && !node.getChildrenNodes().isEmpty()) {
                    BufferedWriter writer = new BufferedWriter(new FileWriter(outputDir+node.getHash()));
                    for (Node child : node.getChildrenNodes()) {// for-Each loop to iterate over all childrens
                        queue.offer(child);
                        writer.append(child.getHash()+ "\n");
                    }
                    writer.close();
                }
            }
            System.out.println();
        }
    }

    static void sendData(Node root) throws IOException {
        if(root == null) return;
        Queue<Node> queue = new LinkedList<>();
        queue.offer(root);
        while(!queue.isEmpty()) {
            int len = queue.size();
            for(int i=0;i<len;i++) { // so that we can reach each level
                Node node = queue.poll();
                System.out.print(node.getEndByte() + " ");
                if(node.isLeaf()){


                }
                if(node.getChildrenNodes()!=null && !node.getChildrenNodes().isEmpty()) {
                    BufferedWriter writer = new BufferedWriter(new FileWriter("/Users/ajinkyarajguru/Documents/CS297/Deliverable3/DecentralizedFileSystem/src/main/resources/fileMetadata/"+node.getHash()));
                    for (Node child : node.getChildrenNodes()) {// for-Each loop to iterate over all childrens
                        queue.offer(child);
                        writer.append(child.getHash()+ "\n");
                    }
                    writer.close();
                }

            }
            System.out.println();
        }
    }
}
